@@ -46,20 +46,23 @@ raw values read out
 """
 
 
-__version__ = '1.3.4'
+__version__ = '1.4.2'
 
-import serial
+import os
 import time
 import argparse
 import sys
 import json
 import signal
 import platform
-import serial.tools.list_ports as list_ports
 from collections import OrderedDict
+import requests
+import serial
+import serial.tools.list_ports as list_ports
+import paho.mqtt.client as mqtt
 
 # Choices allowed for 'get' command
-choices_for_get = [
+CHOICES_FOR_GET = [
     'pressure', 'flow', 'o2', 'bpm', 'backup',
     'tidal', 'peep', 'temperature', 'power_mode',
     'battery', 'version', 'alarm', 'warning',
@@ -70,7 +73,7 @@ choices_for_get = [
 ]
 
 # Choices allowed for 'set' command
-choices_for_set = [
+CHOICES_FOR_SET = [
     'run', 'mode', 'rate', 'ratio',
     'assist_ptrigger', 'assist_flow_min',
     'ptarget', 'pressure_support', 'peep',
@@ -93,6 +96,12 @@ settings_to_store = [
 ]
 """
 
+# Measurements returned from 'get all' command
+MEASUREMENTS_RETURNED = [
+    'p_patient', 'f_total', 'o2', 'bpm', 'v_total', 'peep', 'temp', 'bat_pwr',
+    'bat_charge', 'p_peak', 'v_total_insp', 'v_total_exhl', 'v_minute'
+]
+
 # Verbosity option
 verbose = False
 rate = 10
@@ -101,6 +110,7 @@ terminate = False
 
 
 def convert_json_to_csv(json_file, csv_file):
+    """Convert JSON log to CSV"""
     # Load JSON data in OrderedDict, to preserve current order
     json_data = json.load(json_file, object_pairs_hook=OrderedDict)
 
@@ -109,8 +119,8 @@ def convert_json_to_csv(json_file, csv_file):
     # Compact Format
     if "format" in json_data:
         # Create header
-        for n, elem in enumerate(json_data['format']):
-            csv_file.write("{0}{1}".format('' if n == 0 else ',', elem))
+        for idx, elem in enumerate(json_data['format']):
+            csv_file.write("{0}{1}".format('' if idx == 0 else ',', elem))
         csv_file.write('\n')
 
         # Write out data
@@ -121,14 +131,14 @@ def convert_json_to_csv(json_file, csv_file):
     # Non-compact format
     else:
         if "data" in json_data:
-            for n, (key, value) in enumerate(json_data['data'][0].items()):
-                csv_file.write("{0}{1}".format('' if n == 0 else ',', key))
+            for idx, (key, value) in enumerate(json_data['data'][0].items()):
+                csv_file.write("{0}{1}".format('' if idx == 0 else ',', key))
             csv_file.write('\n')
 
             for row in json_data['data']:
-                for n, (key, value) in enumerate(row.items()):
+                for idx, (key, value) in enumerate(row.items()):
                     csv_file.write("{0}{1}".format(
-                        '' if n == 0 else ',', value))
+                        '' if idx == 0 else ',', value))
                 csv_file.write('\n')
 
 
@@ -157,12 +167,12 @@ def set_log_settings(log_file, verbose=False):
         prev_log = json.load(log_file)
     except Exception as e:
         print(e)
-        exit(-1)
+        sys.exit(-1)
 
     settings_to_load = prev_log["settings"]
     for setting in settings_to_load:
         # Write all settable settings back to ESP32
-        if setting in choices_for_set and setting != "run":
+        if setting in CHOICES_FOR_SET and setting != "run":
             if verbose:
                 print("Setting " + setting + " to " +
                       str(settings_to_load[setting]))
@@ -175,21 +185,25 @@ def set_log_settings(log_file, verbose=False):
 def get_log_settings():
     """Gets the log settings from the device"""
     settings_resp = {}
-    for setting in choices_for_get:
-        if setting in choices_for_set and setting != "run":
+    for setting in CHOICES_FOR_GET:
+        if setting in CHOICES_FOR_SET and setting != "run":
             resp = get_mvm_param(ser, setting)
-            if(resp is not False):
-                # This is dirty, but effective
-                try:
-                    resp = int(resp)
-                except Exception:
-                    resp = float(resp)
+            if resp is not False:
+                # Needed for handling inconsistencies between firmware and mock
+                if resp == "unknown":
+                    resp = "NaN"
+                else:
+                    # This is dirty, but effective
+                    try:
+                        resp = int(resp)
+                    except Exception:
+                        resp = float(resp)
                 settings_resp[setting] = resp
 
     return settings_resp
 
 
-def _parse_response(ser):
+def parse_response(ser):
     """Parse a response and check if its valid from the ESP32, format is
     'valore=...'"""
     # Sometimes the simpliest solution is best
@@ -202,14 +216,14 @@ def _parse_response(ser):
     TIMEOUT = 5.0  # Don't try for longer than this
     num_tries = 0
     start_time = time.time()
-    while(num_tries < MAX_RESP_TRIES and time.time() < (start_time + TIMEOUT)):
+    while num_tries < MAX_RESP_TRIES and time.time() < (start_time + TIMEOUT):
         # Remove the terminator(s)
         # Use read_until() as it should time out
         response = ser.read_until().decode('utf-8').strip()
         try:
-            if(response is not None):
+            if response is not None:
                 check, value = response.split('=')
-                if(check.lower() == 'valore'):
+                if check.lower() == 'valore':
                     return value
         except Exception:
             pass
@@ -222,21 +236,21 @@ def _parse_response(ser):
 def get_mvm_param(ser, param):
     """Request the ESP32 get the value for a given parameter and transmit it"""
     num_tries = 0
-    while(num_tries < 3):
+    while num_tries < 3:
         try:
             request = 'get ' + str(param) + '\r\n'
             ser.write(request.encode('utf-8'))
-            response = _parse_response(ser)
+            response = parse_response(ser)
         except Exception as e:
             print("Error communicating with device")
             print(e)
             return False
 
-        if(response is not False):
-            if(response == 'no_data'):
+        if response is not False:
+            if response == 'no_data':
                 print("Invalid or unknown command")
                 return False
-            elif(response[0:5] == 'ERROR'):
+            if response[0:5] == 'ERROR':
                 if verbose:
                     print("Command Error, Retrying...")
             else:
@@ -250,21 +264,21 @@ def get_mvm_param(ser, param):
 def set_mvm_param(ser, param, value):
     """Request the ESP32 set a parameter to a given value"""
     num_tries = 0
-    while(num_tries < 3):
+    while num_tries < 3:
         try:
             request = 'set ' + str(param) + ' ' + str(value) + '\r\n'
             ser.write(request.encode('utf-8'))
-            response = _parse_response(ser)
+            response = parse_response(ser)
         except Exception as e:
             print("Error communicating with device")
             print(e)
             return False
 
         try:
-            if(response is not False):
-                if (response.lower() == 'ok'):
+            if response is not False:
+                if response.lower() == 'ok':
                     return True
-                elif(verbose):
+                if verbose:
                     print("Command Error, Retrying...")
             else:
                 return False
@@ -283,7 +297,7 @@ def cmd_get(args):
     if result is not False:
         print(result)
     else:
-        exit(-1)
+        sys.exit(-1)
 
 
 def cmd_set(args):
@@ -292,10 +306,21 @@ def cmd_set(args):
 
     result = set_mvm_param(ser, args.param, args.value)
     if result is not False:
-        if(verbose):
+        if verbose:
             print("Success")
     else:
-        exit(-1)
+        sys.exit(-1)
+
+
+def url_retrieve(url, outfile):
+    """Retrieve content from a URL and put it intoa file"""
+    resp = requests.get(url, allow_redirects=True)
+    if resp.status_code != 200:
+        raise ConnectionError('could not download {}\nerror code: {}'
+                              ''.format(url, resp.status_code))
+
+    with open(outfile, "wb+") as dfile:
+        dfile.write(resp.content)
 
 
 def cmd_log(args):
@@ -305,20 +330,84 @@ def cmd_log(args):
     global rate
     global verbose
 
+    # Set up remote monitoring
+    if args.monitor is not None:
+
+        # Get configuration parameters
+        try:
+            with open(args.monitorconfigfile, "r") as mcfgfile:
+                monitorconfig = json.load(mcfgfile)
+        except Exception as e:
+            print("Error")
+            print(e)
+            sys.exit(-1)
+
+        # Set up and connect to the broker
+        stats = {}  # Used by callback functions
+        mqttclient = mqtt.Client(userdata=stats)
+        if not os.path.isfile(monitorconfig["cacertfile"]):
+            url_retrieve(
+                monitorconfig["cacerturl"], monitorconfig["cacertfile"])
+        mqttclient.tls_set(ca_certs=monitorconfig["cacertfile"])
+        mqttclient.username_pw_set(args.monitor[0], args.monitor[1])
+        mqttclient.on_connect = on_connect
+        mqttclient.on_publish = on_publish
+        mqttclient.connect(
+            monitorconfig["broker"], monitorconfig["brokerport"], 60)
+        mqttclient.loop_start()
+        if not len(monitorconfig["institute"]):
+            print("no \"institute\" in %s, "
+                  "please, enter your institute name"
+                  % args.monitorconfigfile)
+            sys.exit(-1)
+        topic = "mvm/%s/%s/%s" % (monitorconfig["institute"],
+                                  args.campaign, args.run)
+        print("\n\tmeasurements are visible at %s" %
+              monitorconfig["dashboard"])
+
+        # Set minimum reading interval to 0.1 s (<10 Hz)
+        if rate < 0.1:  # s
+            print("reducing the MVM readout rate to 10 Hz "
+                  "to avoid overloading the remote server")
+            rate = 0.1  # s
+
+        # Set minimum send interval
+        if args.sendinterval < 5.:  # s
+            print("increasing the data sending interval to 5 s "
+                  "to avoid overloading the remote server")
+            sendinterval = 5.  # s
+        else:
+            sendinterval = args.sendinterval
+
+        # Stats for callback functions
+        stats["topic"] = topic
+        if args.logfile is not sys.stdout or args.nolog:
+            stats["lineend"] = "\r"
+        else:
+            stats["lineend"] = "\r\n"
+        stats["sendinterval"] = sendinterval  # s
+        stats["sentcounter"] = 0
+        stats["datacounter"] = 0  # MB
+
+        print("\t  last data sent at\t   run time  measurements\t succ./  "
+              "sent\t    tot. data\t message size\t      data rate\t\tmvm/"
+              "institute/campaign/run")
+        payload = []
+
     # Disable verbose if logging to stdout
-    if(args.logfile is not sys.stdout and verbose):
+    if args.logfile is not sys.stdout and verbose:
         verbose = True
     else:
         verbose = False
 
     # If the run is active, turn it off and wait a bit
     # so the device can settle before restart
-    if(not args.override_run):
+    if not args.override_run:
         if get_mvm_param(ser, "run") == "1":
             set_mvm_param(ser, "run", 0)
 
     # Should we configure the ESP32 with settings from a previous logfile?
-    if(args.use_cfg is not None):
+    if args.use_cfg is not None:
         set_log_settings(args.use_cfg, verbose)
         # Wait some time for those settings to come into effect
         time.sleep(3)
@@ -327,12 +416,12 @@ def cmd_log(args):
     settings_resp = get_log_settings()
 
     # Dump out the settings to the log file
-    args.logfile.write("{\n  \"settings\": " +
-                       json.dumps(settings_resp, indent=2) + ",\n")
+    if not args.nolog:
+        args.logfile.write("{\n  \"settings\": " +
+                           json.dumps(settings_resp, indent=2) + ",\n")
 
     # Start the data portion of the log file
-
-    if(args.compact):
+    if not args.nolog and args.compact:
         args.logfile.write(
             '\"format\": ['
             '\"time\",'
@@ -350,29 +439,40 @@ def cmd_log(args):
             '\"v_total_exhl\",'
             '\"f_peak\"],\n')
 
-    args.logfile.write("\"data\": [\n")
+    if not args.nolog:
+        args.logfile.write("\"data\": [\n")
     first = True  # Used to print the ',' between entries
 
     # Start run
     set_mvm_param(ser, "run", 1)
+    start_time_offset = time.time()
+    lasttime = start_time_offset
+    if args.monitor is not None:
+        # Prepare settings for payload
+        monitorcounter = 0
+        logcounter = 0
+        payloadsettings = settings_resp
+        payloadsettings["firmware"] = get_mvm_param(
+            ser, "version")  # Firmware version
+        payloadsettings["type"] = "setting"  # Set type label
+        stats["logstarttime"] = start_time_offset
 
-    while(1):
+    while 1:
+
+        # Get all measurements
         resp = get_mvm_param(ser, 'all')
-        if(resp is False):
-            exit(-1)
+        if resp is False:
+            sys.exit(-1)
 
-        if(verbose):
+        if verbose:
             print(resp)
 
-        if(first):
-            start_time_offset = time.time()
-
         # Ugly, but split up the data and use some sort of name that makes
         # sense the variable names used inside the Arduino code are a bit
         # hard to parse/grok
         data_split = resp.split(',')
 
-        if(args.compact):
+        if not args.nolog and args.compact:
             args.logfile.write(
                 '{13}[{14:.3f},{0},{1},{2},{3},{4},'
                 '{5},{6},{7},{8},{9},{10},{11},{12}]\n'.format(
@@ -392,7 +492,7 @@ def cmd_log(args):
                     ' ' if first else ',',
                     0.000 if first else float(time.time()-start_time_offset)))
 
-        else:
+        elif not args.nolog:
             args.logfile.write(
                 '{13}{{"time":{14:.3f},"p_patient":{0},'
                 '"f_total":{1},"o2":{2},"bpm":{3},'
@@ -416,27 +516,107 @@ def cmd_log(args):
                     ' ' if first else ',',
                     0.000 if first else float(time.time()-start_time_offset)))
 
-        if(first):
+        # Remote monitoring
+        if args.monitor is not None:
+
+            # Dump the measurements to the payload
+            payload.append(dict(zip(
+                # Keys
+                MEASUREMENTS_RETURNED+["alarm", "warning",
+                                       "firmware", "type", "timestamp"],
+                # Values
+                json.loads("["+resp+"]")+[
+                    int(get_mvm_param(ser, "alarm")),
+                    int(get_mvm_param(ser, "warning")),
+                    get_mvm_param(ser, "version"),
+                    "measurement",
+                    time.time()])))
+
+            # Send data to the monitoring server every <sendinterval> seconds
+            if (time.time() - lasttime > sendinterval):
+
+                # Dump the settings to the payload
+                payloadsettings["timestamp"] = time.time()
+                payload.append(payloadsettings)
+
+                # Send data
+                monitorcounter += 1
+                payloadsize = len(json.dumps(payload).encode('utf-8'))/1024.
+                stats["payloadsize"] = payloadsize
+                stats["logcounter"] = logcounter
+                stats["monitorcounter"] = monitorcounter
+                mqttclient.publish(topic, json.dumps(payload), qos=2)
+                print_monitoring_stats(stats)
+
+                # At last
+                lasttime = time.time()
+                payload = []
+
+            logcounter += 1
+
+        if first:
             first = False
 
-        if terminate or (
-            (args.time != 0) and time.time() >= (start_time_offset + args.time)
-        ):
-            args.logfile.write("]}\n")
-            if(args.also_csv):
-                args.logfile.seek(0, 0)
-                with open(args.logfile.name[0:-4] + "csv", "w+") as csvfile:
-                    convert_json_to_csv(args.logfile, csvfile)
-
-            args.logfile.close()
-            if(not args.override_run):
+        if (terminate or
+           ((args.time != 0) and
+                time.time() >= (start_time_offset + args.time))):
+            if not args.nolog:
+                args.logfile.write("]}\n")
+                if args.also_csv:
+                    args.logfile.seek(0, 0)
+                    with open(
+                            args.logfile.name[0:-4] + "csv", "w+"
+                    ) as csvfile:
+                        convert_json_to_csv(args.logfile, csvfile)
+                args.logfile.close()
+            if not args.override_run:
                 set_mvm_param(ser, "run", 0)
             print('')
-            exit(0)
+            sys.exit(0)
 
         time.sleep(rate)
 
 
+def on_connect(client, stats, flags, rc):
+    """Callback function used when connection to the broker is established."""
+    if rc != 0:
+        print("could not connect: "+rc)
+
+
+def on_publish(client, stats, mid):
+    """Callback function used when data is successfully sent."""
+    stats["sentcounter"] += 1
+    stats["datacounter"] += stats["payloadsize"]/1024.  # MB
+    print_monitoring_stats(stats)
+
+
+def print_monitoring_stats(stats):
+    """Print monitoring stats on the same line over and over again."""
+    sys.stdout.write("\t%s    %s\t   %s\t%s/%s\t%s MB\t%s kB\t%s kB/s\t\t%s%s" %
+                     (time.strftime("%Y-%m-%d %H:%M:%S"),
+                      get_time_string(time.time()-stats["logstarttime"]),
+                      "{:06}".format(stats["logcounter"]),
+                      "{:06}".format(stats["sentcounter"]),
+                      "{:06}".format(stats["monitorcounter"]),
+                      "{:10.3f}".format(stats["datacounter"]),  # MB
+                      "{:10.1f}".format(stats["payloadsize"]),  # kB
+                      "{:10.1f}".format(stats["payloadsize"] / \
+                                        stats["sendinterval"]),  # kB/s
+                      stats["topic"], stats["lineend"]))
+
+
+def get_time_string(seconds):
+    """Convert seconds into a string of days, hours, minutes and seconds."""
+    d = int(seconds//(60*60*24))
+    seconds = seconds % (60*60*24)
+    h = int(seconds//(60*60))
+    seconds = seconds % (60*60)
+    m = int(seconds//(60))
+    s = round(seconds % (60))
+    return ("%s %s:%s:%s" % ("{:03}".format(d), "{:02}".format(h),
+                             "{:02}".format(m), "{:02}".format(s)))
+
+
 def cmd_console_log(args):
     """Console Log command wrapper"""
     global terminate
@@ -444,24 +624,22 @@ def cmd_console_log(args):
     global verbose
 
     # Disable verbose if logging to stdout
-    if(args.logfile is not sys.stdout and verbose):
-        verbose = True
-    else:
+    if args.logfile is sys.stdout:
         verbose = False
 
     # If the run is active, turn it off and wait a bit
     # so the device can settle before restart
-    if(not args.override_run):
+    if not args.override_run:
         if get_mvm_param(ser, "run") == "1":
             set_mvm_param(ser, "run", 0)
 
     # Should we configure the ESP32 with settings from a previous logfile?
-    if(args.use_cfg is not None):
+    if args.use_cfg is not None:
         set_log_settings(args.use_cfg)
 
         # Wait a bit and clear out the old settings
         time_to_poll = time.time()
-        while(time.time() < (time_to_poll + 3.0)):
+        while time.time() < (time_to_poll + 3.0):
             resp = ser.read_until().decode('utf-8').strip()
             time.sleep(0.01)
 
@@ -472,7 +650,7 @@ def cmd_console_log(args):
     args.logfile.write("{\n  \"settings\": \n" +
                        json.dumps(settings_resp, indent=2) + ",\n")
 
-    if(args.compact):
+    if args.compact:
         args.logfile.write(
             '\"format\": ['
             '\"time\",'
@@ -497,13 +675,13 @@ def cmd_console_log(args):
 
     # Start up console logger
     resp = set_mvm_param(ser, 'console', 1)
-    if(resp is False):
-        exit(-1)
+    if resp is False:
+        sys.exit(-1)
 
-    if(verbose):
+    if verbose:
         print(resp)
 
-    while(1):
+    while 1:
 
         # Ugly, but split up the data and use some sort of name that makes
         # sense the variable names used inside the Arduino code are a bit
@@ -512,15 +690,15 @@ def cmd_console_log(args):
         # Can't use 'get' call here, as console log just spews out messages
         # unrequested
         resp = ser.read_until().decode('utf-8').strip()
-        if(verbose):
+        if verbose:
             print(resp)
 
-        if(first):
+        if first:
             start_time_offset = time.time()
 
         data_split = resp.split(',')
 
-        if(args.compact):
+        if args.compact:
             args.logfile.write(
                 '{11}[{12:.3f},{0},{1},{2},{3},'
                 '{4},{5},{6},{7},{8},{9},{10}]\n'.format(
@@ -558,33 +736,35 @@ def cmd_console_log(args):
                     '' if first else ',',
                     0.000 if first else float(time.time()-start_time_offset)))
 
-        if(first):
+        if first:
             first = False
 
         if terminate or (
-            (args.time != 0) and time.time() >= (start_time_offset + args.time)
+                (args.time != 0) and
+                time.time() >= (start_time_offset + args.time)
         ):
             # Turn off console logging
             set_mvm_param(ser, 'console', 0)
             args.logfile.write("]}\n")
 
-            if(args.also_csv):
+            if args.also_csv:
                 args.logfile.seek(0, 0)
                 with open(args.logfile.name[0:-4] + "csv", "w+") as csvfile:
                     convert_json_to_csv(args.logfile, csvfile)
 
             args.logfile.close()
 
-            if(not args.override_run):
+            if not args.override_run:
                 set_mvm_param(ser, "run", 0)
             print('')
-            exit(0)
+            sys.exit(0)
 
         # Brief sleep so this script doesn't lock up CPU
         time.sleep(0.005)
 
 
 def cmd_convert(args):
+    """Command to convert JSON log file to CSV"""
     convert_json_to_csv(args.jsonfile, args.csvfile)
 
 
@@ -626,7 +806,6 @@ def main():
             choices=[1, 0],
             help="Force DTR to 1 or 0"
         )
-
         subparsers = parser.add_subparsers(
             title='Subcommands',
             help='Commands available')
@@ -654,8 +833,8 @@ def main():
             usage="%(prog)s <param>")
         parser_get.add_argument(
             "param",
-            choices=choices_for_get,
-            help=' '.join(choices_for_get),
+            choices=CHOICES_FOR_GET,
+            help=' '.join(CHOICES_FOR_GET),
             metavar='param')
         parser_get.set_defaults(func=cmd_get)
 
@@ -666,8 +845,8 @@ def main():
             help="set <param> <value>")
         parser_set.add_argument(
             "param",
-            choices=choices_for_set,
-            help=' '.join(choices_for_set),
+            choices=CHOICES_FOR_SET,
+            help=' '.join(CHOICES_FOR_SET),
             metavar="param")
         parser_set.add_argument(
             "value",
@@ -711,6 +890,12 @@ def main():
             '--compact',
             action='store_true',
             help="Generate compact log")
+        parser_log.add_argument(
+            '-n',
+            '--no-log',
+            dest='nolog',
+            action='store_true',
+            help="Do not print to screen nor save to file")
         parser_log.add_argument(
             '-o',
             '--override-run',
@@ -742,6 +927,38 @@ def main():
             type=argparse.FileType('w+'),
             default=sys.stdout,
             help="Optional, leave blank for stdout")
+        parser_log.add_argument(
+            '-m',
+            '--monitor',
+            nargs=2,
+            metavar=('username', 'password'),
+            default=None,
+            help="Client username and password to"
+            " send data for remote monitoring")
+        parser_log.add_argument(
+            '-mc',
+            '--monitor-config',
+            metavar="<file>",
+            dest="monitorconfigfile",
+            default="monitorconfig.json",
+            type=str,
+            help="Remote monitoring configuration file")
+        parser_log.add_argument(
+            '--campaign',
+            default=time.strftime("%Y%m%d"),
+            metavar="<campaign>",
+            help="Campaign name")
+        parser_log.add_argument(
+            '--run',
+            default="test",
+            metavar="<run>",
+            help="Run name")
+        parser_log.add_argument(
+            '--sendinterval',
+            default=5.,
+            metavar="<sendinterval>",
+            type=float,
+            help="Time interval to send monitoring data")
         parser_log.set_defaults(func=cmd_log)
 
         # 'Console' Log command
@@ -794,14 +1011,14 @@ def main():
 
     except Exception as e:
         print(e)
-        exit(-1)
+        sys.exit(-1)
 
     # Try to establish connection with ESP32
     try:
-        if(args.port is None and len(get_available_serial_ports()) > 0):
+        if args.port is None and len(get_available_serial_ports()) > 0:
             args.port = get_available_serial_ports()[0]
 
-        if(args.force_dtr is None):
+        if args.force_dtr is None:
             plt = platform.system()
             if plt == "Windows":
                 rts = False
@@ -827,13 +1044,13 @@ def main():
         ser.dtr = 1 if dtr is True else 0
         ser.open()
     except Exception:
-        if(args.port is None):
+        if args.port is None:
             print("No serial port available, is the cable unplugged?")
         else:
             print("Failed to connect to serial port " + args.port)
             print("Available serial ports:")
             print(' \n'.join(get_available_serial_ports()))
-        exit(-1)
+        sys.exit(-1)
 
     # Run requested subcommand function
     # This is a fix for python3 + argparse...
@@ -844,7 +1061,7 @@ def main():
 
     func(args)
 
-    exit(0)
+    sys.exit(0)
 
 
 if __name__ == "__main__":
